library(nhdplusTools)
library(dplyr)
library(sf)
library(hyRefactor)
library(tidyr)
library(hyfabric)
library(mapview)
library(terra)
library(sbtools)
library(jsonlite)
library(fst)

rpu_vpu <- readr::read_csv(system.file("extdata/rpu_vpu.csv", package = "hyfabric"), 
                           show_col_types = FALSE)
rpu_vpu_out <- readr::read_csv("cache/rpu_vpu_out.csv", 
                               col_types = c("c", "c", "i" , "i"), show_col_types = FALSE)

if(nchar(Sys.getenv("HYDREG")) > 1) {
  vpu_codes <- Sys.getenv("HYDREG")
}

if(!exists("rpu_codes")) {
  if(exists("vpu_codes")) {
    rpu_codes <- rpu_vpu[rpu_vpu$vpuid %in% vpu_codes, ]
    rpu_code <- rpu_codes$rpuid[1]
  } else {
    vpu_codes <- get_rpu_dependent_vars()
    rpu_code <- vpu_codes$r
    vpu_codes <- vpu_codes$v
    rpu_codes <- rpu_vpu[rpu_vpu$vpuid %in% vpu_codes, ]
  }
}

if(!exists("vpu_codes")) {
  vpu_codes <- get_rpu_dependent_vars(rpu_code)$v
}

if(!nchar(out_agg_gpkg <- Sys.getenv("OUT_GPKG")) > 5) 
  out_agg_gpkg <- file.path("cache", paste0(rpu_code, "_aggregate.gpkg"))

if(!nchar(out_refac_gpkg <- Sys.getenv("OUT_REFACTOR_GPKG")) > 5)
  out_refac_gpkg <- file.path("cache", paste0(rpu_code, "_refactor.gpkg"))

if(!nchar(cache_split <- Sys.getenv("CACHE_SPLIT")) > 5) 
  cache_split <- file.path("temp", paste0(rpu_code, ".rda"))

if(!nchar(lookup_table_file <- Sys.getenv("LOOKUP_TABLE")) > 5) 
  lookup_table_file <- file.path("cache", paste0(rpu_code, "_lookup.csv"))

if(!nchar(vpu_lookup_table_file <- Sys.getenv("VPU_LOOKUP_TABLE")) > 5) 
  vpu_lookup_table_file <- file.path("cache", paste0("lookup_", vpu_codes, ".csv"))


options(scipen = 9999)

# Source Data
data_paths <- jsonlite::read_json(file.path("cache", "data_paths.json"))

#nhdplus_path(data_paths$nhdplus_gdb)
#suppressWarnings(staged_nhd <- stage_national_data())

# Defined and used broadly
nhd_flowline <- "reference_flowline"
nhd_catchment <- "reference_catchment"
nhd_waterbody <- "reference_waterbody"
nhd_outlet <- "nhd_outlet"
nhd_network <- "reference_network"

# Unit conversions 
m_per_km <- 1000
ft_per_km <- 3280.84 # feet per kilometer
s_per_hr <- 3600 # seconds per hour

# Intermediate layers created during 02_Navigate
xwalk_layer <- paste0("HUC12_nhd") # HUC12 - nhdcat crosswalk, built in Nav for VPU 20
nav_poi_layer <- paste0("POIs_tmp_", vpu_codes) # Rolling Nav POI layer added to/modified througout nav workflow
WBs_layer <-  paste0("WB_", vpu_codes) # Waterbodies within VPU
WB_events_layer <- paste0("WB_events_", vpu_codes) # inlet and outlet events for NHDArea and HR waterbodies
poi_moved_layer <- paste0("POIs_mv_", vpu_codes) # POIs moved from original COMID assignment
nsegments_layer <- paste0("nsegment_", vpu_codes) # Minimally-sufficient network dissolved by POI_ID
pois_all_layer <- paste0("POIs_", vpu_codes) # All POIs binded together
poi_xwalk_layer <- paste0("poi_xwalk_layer_", vpu_codes) # POIs that changed COMIDS during the navigate part of the workflow
final_poi_layer <- "POIs"
dup_pois <- "dup_POIs"

poi_data_table <- "poi_data"
poi_geometry_table <- "poi_geometry_table"

event_table <- "event_data"
event_geometry_table <- "event_geometry_table"

# Created during refactor workflow
refactored_layer <- "collapsed"
reconciled_layer <- "refactored_flowpaths"
split_layer <- "split_events"
divide_layer <- "refactored_divides"
split_divide_layer <- "split_divides"
agg_fline_layer <- "aggregated_flowpaths"
agg_cats_layer <- "aggregated_divides"
outlets_layer <- "outlets"
mapped_outlets_layer <- "mapped_POIs"
lookup_table_refactor <- "lookup_table"
catchment_network_table <- "catchment_network"

# output  geopackage file names
ref_gpkg <- file.path(data_paths$ref_fab_path, paste0(vpu_codes, "_reference_features.gpkg"))
nav_gpkg <- file.path("cache", paste0("reference_", vpu_codes,".gpkg"))
temp_gpkg <- file.path("cache", paste0("nav_", vpu_codes,".gpkg"))

rfc_gpkg <- file.path("cache", paste0("refactor_", vpu_codes, ".gpkg"))

gf_gpkg <- file.path("cache", paste0("GF_", vpu_codes, ".gpkg"))
gf_gpkg_conus <- "cache/reference_CONUS.gpkg"

gage_info_gpkg <- "cache/Gages_info.gpkg"
gage_info_csv <- "cache/Gages_info.csv"
gage_info_csvt <- "cache/Gages_info.csvt"

# Defined during NonDend.Rmd
ND_gpkg <- file.path("temp", paste0("ND_", vpu_codes,".gpkg"))

divides_xwalk <- "divides_nhd"
HRU_layer <- "all_cats"
divides_nd <- "divides_nd"
missing_terms <- "miss_terms"
missing_cats <- "miss_cats"
ND_table <- "ND_table"
